\\ Copyright 2015, 2016 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\-----------------------------------------------------------------------------
\\ continued fraction of real root of a polynomial, so any algebraic number

contfrac_polroot(p,nmax,lo,hi) =
{
  my(x=variable(p));
  my(lo_sign = sign(subst(p,x,lo)),
     hi_sign = sign(subst(p,x,hi)));
  if(lo_sign * hi_sign != -1,
     error("p() of lo=",lo," hi=",hi," not opposite signs"));

  my(l = List([]));
  for(i=1,nmax,
     \\ print("p "p" range "lo" to "hi"  root ",polroots(p)[1]);
     \\ print(gcd(Vec(p)));

     if(p==0,break());
         
     \\ consistency checks, comment out for speed
     if(! (hi > lo),
        error("oops, don't have hi=",hi," > lo=",lo));
     if(lo_sign * sign(subst(p,x,lo)) < 0,
        error("oops, lo_sign wrong"));
     if(hi_sign * sign(subst(p,x,hi)) < 0,
        error("oops, lo_sign wrong"));

     \\ Shift hi,lo to integers if possible, in order to avoid
     \\ expression swell.  If ceil(lo) happens to be above the root then
     \\ can't raise lo, but can reduce hi the same as for the binary
     \\ search.  Similarly if floor(hi) happens to be below the root.
     my(mid = ceil(lo));
     if(hi > mid,
        if(sign(subst(p,x,mid)) == lo_sign, lo = mid, hi=mid));
     mid = floor(hi);
     if(mid > lo,
        if(sign(subst(p,x,mid)) == lo_sign, lo = mid, hi=mid));

     \\ binary search between lo and hi for integer part of root,
     \\ stop when lo to hi have same integer part
     while(floor(hi) > floor(lo),
           mid = (hi+lo)/2;
           \\ print(" at "lo"["lo_sign"] to "hi"["hi_sign"]"
           \\       " consider "mid"["sign(subst(p,x,mid))"]");
           if(sign(subst(p,x,mid)) == lo_sign, lo = mid, hi=mid));

     my(a = floor(lo));
     listput(l,a);
     \\ print("quotient a="a"  at v["i"]");

     p = subst(p,x, x+a);
     lo -= a;
     hi -= a;
     \\ print(" shift to p "p"  "lo" to "hi"  root ",polroots(p)[1]);

     \\ if lo or hi are zero cannot invert them, so binary search to
     \\ tighten the bounds until not zero
     while(lo == 0 || hi == 0,
           mid = (hi+lo)/2;
           my(mid_sign = sign(subst(p,x,mid)));
           if(mid_sign == 0, break(2));
           if(mid_sign == lo_sign, lo = mid, hi=mid));
     
     \\ p = subst(p,x, 1/x) * x^poldegree(p);
     p = Polrev(Vec(p));
     my(new_lo = 1/hi, new_hi = 1/lo);
     lo = new_lo; hi = new_hi;
     lo_sign = - lo_sign;
     hi_sign = - hi_sign);
  Vec(l);
}
{
addhelp(contfrac_polroot,
"v = contfrac_polroot(p,nmax, lo,hi)
Return a vector which is the first nmax terms of continued fraction of a
real root of p.

lo and hi are rationals which are upper and lower bounds for the desired
root.  There must be just one root in that range and it must not be a
repeated root, so p(lo) and p(hi) must have opposite signs.

The first continued fraction term is the integer part, possibly 0.  The
vector returned is the first nmax many terms, except if the root is rational
and its continued fraction terminates sooner.")};