\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.


read("c-curve-defines.gp");

check_count = 0;
check_exact(got,want, name="") =
{
  check_count++;
  if(got!= want,
     print("\nwrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\ 1,2,3,4,5,6,7,8 cycle, count runs
vector_cycle_count_run_1s(v) = \
  if(#v==0,0, sum(i=1,#v-1, v[i+1] > v[i]) + (v[1] > v[#v]));
check_exact(vector_cycle_count_run_1s([0,1,1,0,1,1]), 2);
check_exact(vector_cycle_count_run_1s([0,1,1,1,1,1]), 1);
check_exact(vector_cycle_count_run_1s([1,1,1]), 0);

conf_count_turn_end(conf) =
{
  my(v = conf[1..8], runs = vector_cycle_count_run_1s(v));
  if(runs, 4*runs - vecsum(v));
}
check_exact(conf_count_turn_end(conf_make([1])), 3);
check_exact(conf_count_turn_end(conf_make([1,2])), 2);
check_exact(conf_count_turn_end(conf_make([8,1])), 2);
check_exact(conf_count_turn_end(conf_make([1,2,3])), 1);
check_exact(conf_count_turn_end(conf_make([1,2,3,4])), 0); \\ straight
check_exact(conf_count_turn_end(conf_make([7,8,1,2])), 0); \\ straight
check_exact(conf_count_turn_end(conf_make([1,2,3,4,5])), -1);
check_exact(conf_count_turn_end(conf_make([1,2,3,4,5,6])), -2);
check_exact(conf_count_turn_end(conf_make([1,2,3,4,5,6,7])), -3);
check_exact(conf_count_turn_end(conf_make([1, 3])), 3+3, "two +135");

conf_keep_1to8(conf) = concat(conf[1..8], vector(7));

conf_rotate(conf) = [ conf[3], conf[4], conf[5], conf[6], \
                      conf[7], conf[8], conf[1], conf[2], 0,0,0, 0,0,0,0];

conf_rotate_to_min(conf) =
{
  conf = conf_keep_1to8(conf);
  my(v = [ conf,
          conf_rotate(conf),
          conf_rotate(conf_rotate(conf)),
          conf_rotate(conf_rotate(conf_rotate(conf)))
         ]);
  v = vecsort(v,cmp);
  v[#v];
}

\\-----------------------------------------------------------------------------
\\ V1  2, 3, 6
\\ V2  0, 4, 8
\\ V3  -2, +2

conf_count_visits_left(conf) = conf[1] + conf[3] + conf[5] + conf[7];
{
  for(visits=1,3,
     my(seen = Map());
     for(i=1,#conf_occurring_vector,
        my(conf = conf_occurring_vector[i]);
        if(conf_count_visits_left(conf) != visits, next());
        
        my(turn = conf_count_turn_end(conf));
        if(!mapisdefined(seen,turn),
           mapput(seen, turn, 1);
           print("visits "visits"  turn = "turn);
           conf_print(conf_rotate_to_min(conf));
           );
     ));
}
quit;

\\-----------------------------------------------------------------------------
\\ rotations developing even points
\\ V1  turns 2 or 6  as adjacent or not
\\ V2  turns 0, 8 
\\ V3  turns +/- 2

{
  conf = conf_make([1,2, 3,4]);
  conf = conf_rotate_to_min(conf);
  seen = Map();
  while(! mapisdefined(seen,conf),
        mapput(seen,conf,1);
        print("end turn ",conf_count_turn_end(conf));
        conf_print(conf);

        conf = conf_permute(conf,conf_left_permutation);
        conf = conf_mirror(conf);
        \\ print("permute to"); conf_print(conf);

        conf = conf_rotate_to_min(conf);
       );
  print("stop at"); conf_print(conf);
}
quit



\\-----------------------------------------------------------------------------


conf_count_turn_middle(conf) =
{
  my(v = [conf[1],conf[8],conf[9],conf[10]],
     runs = vector_cycle_count_run_1s(v));
  if(runs, 4*runs - 2*vecsum(v));
}
check_exact(conf_count_turn_middle(conf_make([1])),  2); \\ +90
check_exact(conf_count_turn_middle(conf_make([1,8])),  0); \\ straight
check_exact(conf_count_turn_middle(conf_make([1,8,9])), -2); \\ -90

check_exact(conf_count_turn_middle(conf_make([1,9])), 2*2); \\ 2x +90

conf_polygons_net_end(conf) =
{
  my(reps = conf[1]+conf[3]+conf[5]+conf[7]);
  if(0<reps && reps<8, conf_count_turn_end(conf) / reps);
}
conf_polygons_net_middle(conf) =
{
  my(reps = conf[1]+conf[8]+conf[9]+conf[10]);
  if(0<reps && reps<4, conf_count_turn_middle(conf) / reps);
}
conf_polygons_net(conf) = \
  conf_polygons_net_end(conf) + conf_polygons_net_middle(conf);

check_exact(conf_polygons_net_end(conf_make([1])), 3);
check_exact(conf_polygons_net_middle(conf_make([1])), 2);
check_exact(conf_polygons_net(conf_make([1])), 5);

gPolygons_net(x) = {
  3*gOnes(x) +
  sum(i=1,#conf_occurring_vector,
      my(conf = conf_occurring_vector[i]);
      if(conf[1],
         conf_to_gf(conf) * conf_polygons_net(conf)));
}
gPolygons_net = memoize(gPolygons_net);

gPolygons(x) = gPolygons_net(x) / 8;

check_exact(gf_terms(gPolygons_net(x), 4), [8, 2*8, 4*8, 7*8], "net");
check_exact(gf_terms(gPolygons(x), 4), [1, 2, 4, 7], "polygons");

PolygonsPoly = gf_to_linear_recurrence_poly_func(gPolygons(x));
check_exact(PolygonsPoly(x), poly8alt(x) * poly9(x) * poly18(x));

Polygons_generated_samples = [1, 2, 4, 7, 13, 25, 49, 95, 183, 353, 685, 1320, 2532, 4865, 9355, 17975, 34529, 66401, 127753, 245938, 473582];
check_exact(gf_terms(gPolygons(x), #Polygons_generated_samples), \
            Polygons_generated_samples, "generated samples");

\\ pol_ascending_print(pol_partial_fractions(gV1(x)))
pol_ascending_print(pol_partial_fractions(gPolygons(x)))
factor(PolygonsPoly(x))

quit





\\-----------------------------------------------------------------------------

\\ 1,2,3,4,5,6,7,8 cycle, count runs
vector_cycle_count_changes(v) = \
  if(#v==0,0, sum(i=1,#v-1, v[i]!=v[i+1]) + (v[#v] != v[1]));

conf_count_polygon_degree(conf) = {
  if(conf[1],
     vector_cycle_count_changes(conf[1..8]) / 2);
}

gPolygons(x) = {
  gOnes(x) +
  sum(i=1,#conf_occurring_vector,
      my(conf = conf_occurring_vector[i],
         deg = conf_count_polygon_degree(conf),
         visits = conf[1]+conf[3]+conf[5]+conf[7]);
      if(conf[1],
         conf_to_gf(conf) * deg / visits));
}
gPolygons = memoize(gPolygons);

{
  my(k=3);
  for(i=1,#conf_occurring_vector,
     my(conf = conf_occurring_vector[i]);
     my(gf = conf_to_gf(conf));
     my(count = gf_term(gf,k));
     if(count==0, next());

     my(deg = conf_count_polygon_degree(conf));
     if(deg==0, next());
     my(visits = conf[1]+conf[3]+conf[5]+conf[7]);

     print("deg="deg" count="count"  visits="visits);
     conf_print(conf);
  );
  my(polygons = gf_term(gPolygons(x),k));
  print("polygons ",polygons);
}
quit

     my(d=conf_count_polygon_degree(conf));

%                  if(!mapisdefined(m,d),mapput(m,d,0)); \
%                  mapput(m,d, mapget(m,d)+1)); m

% gf_terms(gPolygons(x),14)
% Polygons_generated_samples[1..14]*2
