\\ Copyright 2015, 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.


default(recover,0);
read("c-curve-defines.gp");

check_count = 0;
check_exact(got,want, name="") =
{
  check_count++;
  if(got!= want,
     print("\nwrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count"  "name));
}


\\-----------------------------------------------------------------------------
\\ increment

\\ vertical when dir(n) odd

dz_to_Sd(dz,d) =
{
  1/2*(1 + real( dz*(-I)^d ) - abs( imag(dz*(-I)^d) ) );  \\ [1,0,0,0]
  real( dz*(-I)^d );        \\ [1,0,-1,0]
  imag( dz*(-I)^d );        \\ [-1,0,1,0]
  abs(imag(dz*(-I)^d));     \\ [1,0,1,0]
  real(dz*b*(-I)^d);        \\ [1,1, -1,-1]
  2*real(dz*b*(-I)^d)+2;    \\ [4,4,0,0]
  2*imag(dz*b*(-I)^d)+2;    \\ [4,0,0,4]
  abs(imag(dz*b*(-I)^d)+1); \\ [2,0,0,2]
}
vector(4,d,d--; dz_to_Sd(1,d))
vector(4,d,d--; dz_to_Sd(I,d))
vector(4,d,d--; dz_to_Sd(-1,d))
vector(4,d,d--; dz_to_Sd(-I,d))
\\ {
\\ print("real ",apply(real, [1,I,-1,-I]));
\\ print("imag ",apply(imag, [1,I,-1,-I]));
\\ apply(dz->dz_to_Sd, [1,I,-1,-I])
quit




\\-----------------------------------------------------------------------------

StoN_by_count(n,d) = sum(i=0,n-1, dir(i)%4==d);

  # N= 2^k0 + 2^k1 + 2^k2
  # my(d=1); [Skd(2,d), Skd(1,d-1), Skd(0,d-2)]
  # my(d=1); Skd(2,d) + Skd(1,d-1) + Skd(0,d-2)

StoN_by_bits(n,d) = \
  my(v=binary(n),total=0); \
  for(i=1,#v,if(v[i], \
      total+=Skd(#v-i,d); \
      d--)); \
  total;

\\ {
\\   for(n=0,10,
\\      print("n="n
\\            "  "apply(d->StoN_by_bits(n,d),[0,1,2,3])
\\            "  "apply(d->StoN_by_count(n,d),[0,1,2,3])));
\\ }

check_exact(matrix(100,4,n,d,d--; StoN_by_bits(n,d)), \
            matrix(100,4,n,d,d--; StoN_by_count(n,d)), \
           "StoN_by_bits");


StoN_by_powers(n,orig_d) =
{
  my(v=binary(n),total=n-n%2,d=orig_d); 
  for(i=1,#v-1,if(v[i], 
      my(k=#v-i); 
      total += 2*b^k*(-I)^d; 
      d--)); 
  total=real(total); 
  total /= 4; 
  if(v[#v],total+=(d%4==0)); 
  total;
}
check_exact(matrix(100,4,n,d,d--; StoN_by_powers(n,d)), \
            matrix(100,4,n,d,d--; StoN_by_count(n,d)), \
           "StoN_by_powers");


StoN_by_powers(n,orig_d) =
{
  my(v=binary(n),total=n-n%2,d=orig_d); 
  \\ for(i=1,#v-1,if(v[i], 
  \\     my(k=#v-i); 
  \\     total += 2*b^k*(-I)^d; 
  \\     d--)); 
  total += 2*conj(c_curve_point(n-n%2))*I^d;
  total=real(total); 
  total /= 4; 
  d=orig_d - hammingweight(n-n%2);
  if(v[#v],total+=(d%4==0)); 
  total;
}
check_exact(matrix(100,4,n,d,d--; StoN_by_powers(n,d)), \
            matrix(100,4,n,d,d--; StoN_by_count(n,d)), \
           "StoN_by_powers 2");

StoN_by_powers(n,d) =
{
  my(total=n); 
  total += 2*c_curve_point(n)*(-I)^d;
  total=real(total);
  total += - (n%2)*(-1)^dir(n)*(-1)^d;
  total /= 4; 
  total;
}
check_exact(matrix(100,4,n,d,d--; StoN_by_powers(n,d)), \
            matrix(100,4,n,d,d--; StoN_by_count(n,d)), \
           "StoN_by_powers 3");

\\----------

check_exact(vector(10,n, c_curve_point(n-1) + I^dir(n-1)), \
            vector(10,n, c_curve_point(n)));

\\ n odd [-1, -1, 3, -1] or rotation
\\ n even [0, 2, 0, -2] or rotation
StoN_by_powers(n,d) =
{
  my(total=n); 
  total += 2*c_curve_point(n-1)*(-I)^d;
  total +=     2 * I^dir(n-1) * (-I)^d;
  total += (n%2) * (-1)^dir(n-1) * (-1)^d;
  total=real(total);
   total /= 4; 
  total;
}
{
  forstep(n=0,20,2,
     my(by_powers=apply(d->StoN_by_powers(n,d),[0,1,2,3]));
     my(by_count=apply(d->StoN_by_count(n,d),[0,1,2,3]));
     print("n="n"  "by_powers"  "by_count"  "by_powers-by_count));
}
check_exact(matrix(500,4,n,d,d--; StoN_by_powers(n,d)), \
            matrix(500,4,n,d,d--; StoN_by_count(n,d)), \
           "StoN_by_powers 4");

vector(16,d, real(2*(-I)^d +(-1)^d))
quit

\\--------------------------------------------
StoN_by_powers(n,d) = \
  my(v=binary(n),total); \
  total = n + 2*real(conj(c_curve_point(n))*I^d); \
  if(n%2,total++); \
  total /= 4; \
  total;
for(n=1,12,print("n="n"  "apply(d->StoN_by_powers(n,d),[0,1,2,3])"  "apply(d->StoN_by_count(n,d),[0,1,2,3])))
quit
check_exact(matrix(10,4,n,d,d--; StoN_by_powers(n,d)), \
            matrix(10,4,n,d,d--; StoN_by_count(n,d)));


  #
  # for(n=1,16,print("n="n"  "apply(d->StoN_by_powers(n,d),[0,1,2,3])"  "apply(d->StoN_by_count(n,d),[0,1,2,3])))


  # GP-DEFINE  StoN_b_part(n,d) = \
  # GP-DEFINE    my(v=binary(n),total=0); \
  # GP-DEFINE    for(i=1,#v,if(v[i], \
  # GP-DEFINE        my(k=#v-i); \
  # GP-DEFINE        total += 2*b^k*(-I)^d; \
  # GP-DEFINE        d--)); \
  # GP-DEFINE    total;
  # for(n=1,12,print("n="n"  "StoN_b_part(n,0)"   "2*c_curve_point(n)));
