#!/usr/bin/perl -w

# Copyright 2016, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'xy_rotate_minus90','PartN','transform_to_horizontal','chamfer';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 9;  # of the two joining curves

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::CCurve->new;

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level+1);
}

$Printwrap = 2;
my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my $n_middle = $n_hi / 2;
### $n_lo
### $n_middle
### $n_hi

my $n_start = PartN($level+1);
my $n_end = $n_hi - $n_start;

my $segments = MyLineSegments->new;
my @DJ;
foreach my $n ($n_start .. $n_end-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);

  my @n_list = $path->xyxy_to_n_list_either($x,$y, $x2,$y2);
  @n_list = grep {$_ < $n_hi} @n_list;
  if (@n_list == 2
      && $n_list[0] < $n_middle
      && $n_list[1] >= $n_middle) {
    if ($n == $n_list[0]) {
      push @DJ, [$x,$y, $x2,$y2];
    }
  } else {
    $segments->add($x,$y, $x2,$y2);
  }
}
### @DJ

$Printwrap = 2;
printwrap("\\draw[my grey] ");
$Printwrap += 2;
$segments->printwrap (transform => \&transform);
printwrap(";\n");

$Printwrap = 2;
printwrap("\\draw");
$Printwrap += 2;
foreach my $dj (@DJ) {
  foreach my $s (1,-1) {
    my ($x,$y, $x2,$y2) = @$dj;

    ($x,$y) = transform($x,$y);
    ($x2,$y2) = transform($x2,$y2);
    my ($dx,$dy) = ($x2-$x, $y2-$y);
    $dx *= $s;
    $dy *= $s;
    ($dx,$dy) = xy_rotate_minus90($dx,$dy);

    $x  .= chamfer($dx, '\\MyGap');
    $y  .= chamfer($dy, '\\MyGap');
    $x2 .= chamfer($dx, '\\MyGap');
    $x2 .= chamfer($dy, '\\MyGap');
    printwrap(" ($x,$y)--($x2,$y2)");
  }
}
printwrap(";\n");

# printwrap("\\node (start) at (0,0) [my dot];\n");
{
  my ($x,$y) = $path->n_to_xy($n_start);
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (start) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_end);
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (end) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_middle);
  ($x,$y) = transform($x,$y);
  print "  \\node (middle) at ($x,$y) [my dot];\n";
}

my $DJ = scalar(@DJ);
print "  \\newcommand{\\MyDJ}{$DJ}\n";
print "% GP-Test-Last  DJ($level) == $DJ\n";
print "  % end generated\n";
exit 0;
