#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'transform_to_horizontal','PartN';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::CCurve->new;

my $level = 14;
if (@ARGV) { $level = shift @ARGV; }

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my $n_start = PartN($level);
my $n_end = PartN($level+1);

my $segments = MyLineSegments->new;
foreach my $n ($n_start .. $n_end-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($x,$y) = transform($x,$y);
  ($dx,$dy) = transform($dx,$dy);
  $segments->add($x,$y, $x+$dx,$y+$dy);
}

$Printwrap = 2;
printwrap("\\draw[my line] ");
$Printwrap = 4;
$segments->printwrap;
printwrap(";\n");
my $draw_count = $segments->lines_joined_count_points;

{
  my ($x,$y) = $path->n_to_xy($n_hi / 2);
  ($x,$y) = transform($x,$y);
  print "  \\node at ($x,$y) [my dot,name=middle];\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  $x /= 2;
  $y /= 2;
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (base) at ($x,$y);\n";
}


print "  % total drawn $draw_count points\n";
print "  % end generated\n";
exit 0;
