#!/usr/bin/perl -w

# Copyright 2015, 2016, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'transform_to_horizontal','xy_rotate_minus90';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 3;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::CCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

sub Three_num_neighbours_of_n {
  my ($n) = @_;
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($x2,$y2) = ($x+$dx, $y+$dy);
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);

  my @n_list = ($path->xyxy_to_n($x2,$y2, $x,$y),
                $path->xyxy_to_n($x+$rx,$y+$ry, $x,$y),
                $path->xyxy_to_n($x2,$y2, $x2+$rx,$y2+$ry));
  ### @n_list
  @n_list = grep { defined $_ && $_ < $n_hi } @n_list;
  ### neighbours: "$n is ".scalar(@n_list)
  return scalar(@n_list);
}

sub triangle {
  my ($x,$y, $dx,$dy) = @_;
  ($x,$y) = transform($x,$y);
  ($dx,$dy) = transform($dx,$dy);
  printwrap("\\pic at ($x,$y) [transform shape] {my triangle={$dx,$dy}};");
}

print "  \\begin{scope}[my grey]\n";
$Printwrap = 4;
foreach my $n ($n_lo .. $n_hi-1) {
  next unless Three_num_neighbours_of_n($n) == 0;
  triangle($path->n_to_xy($n),
           $path->n_to_dxdy($n));
}
printwrap_newline();
print "  \\end{scope}\n";

my $Three1 = 0;
$Printwrap = 2;
foreach my $n ($n_lo .. $n_hi-1) {
  next unless Three_num_neighbours_of_n($n) == 1;
  triangle($path->n_to_xy($n),
           $path->n_to_dxdy($n));
  $Three1++;
}
printwrap_newline();
print "  \\newcommand{\\MyThreeOne}{$Three1}\n";

{
  my ($x,$y) = (0,0);
  print "  \\coordinate (start) at ($x,$y);\n";
}
{
  my ($x,$y) = transform($x_hi,$y_hi);
  print "  \\coordinate (end) at ($x,$y);\n";
}

print "  % end generated\n";
exit 0;
