#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Lingua::EN::Numbers;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'chamfer','transform_to_horizontal';
use MyPrintwrap ':all';


my @levels = (0 .. 5);
if (@ARGV) { @levels = split /,/, shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::CCurve->new;

sub is_touch {
  my ($n, $n_hi) = @_;
  my ($x,$y) = $path->n_to_xy($n);
  foreach my $n_other ($path->xy_to_n_list($x,$y)) {
    next if $n_other == $n;
    next if $n_other > $n_hi;
    return 1;
  }
  return 0;
}

sub is_overlap {
  my ($n, $n_hi) = @_;
  my $n_other = $path->xyxy_to_n($path->n_to_xy($n+1), $path->n_to_xy($n));
  return defined($n_other) && $n_other < $n_hi;
}

foreach my $level (@levels) {
  my $transform = sub {
    my ($x,$y) = @_;
    return transform_to_horizontal($x,$y, $level);
  };
  my $MyOverlapGap = '\\MyOverlapGap'
    . ucfirst(Lingua::EN::Numbers::num2en($level));

  $Printwrap = 2;
  my $rotate = (180 - 45*$level) % 360;
  printwrap("\\begin{scope}[shift=(pos-$level)"
            . ($level % 2 ? ",scale={1/sqrt(2)}" : "")
            . "]\n");
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);

  $Printwrap = 4;
  printwrap("\\node (start-$level) at (0,0) [my dot];\n");
  {
    my ($x,$y) = $path->n_to_xy($n_hi);
    ($x,$y) = $transform->($x,$y);
    printwrap("\\node (end-$level) at ($x,$y) [my dot];\n");
  }
  my $arrow = ''; # ($level==0 ? '->,' : '');
  printwrap("\\draw[${arrow}my line] (start-$level)");
  $Printwrap = 6;
  my @overlap_circles;
  foreach my $n ($n_lo+1 .. $n_hi-1) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my ($prev_dx,$prev_dy) = $path->n_to_dxdy($n-1);

    ($x,$y) = $transform->($x,$y);
    ($dx,$dy) = $transform->($dx,$dy);
    ($prev_dx,$prev_dy) = $transform->($prev_dx,$prev_dy);

    # if (is_touch($n,$n_hi)) {
    #   my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
    #   my ($tdx,$tdy) = $path->n_to_dxdy($n);
    #   my $sx = $x . chamfer(-$sdx);
    #   my $sy = $y . chamfer(-$sdy);
    #   my $tx = $x . chamfer($tdx);
    #   my $ty = $y . chamfer($tdy);
    #   printwrap("--($sx,$sy)");
    #   printwrap("--($tx,$ty)");

    if (is_overlap($n,$n_hi) && is_overlap($n-1,$n_hi)) {
      # middle of an overlap
      my ($sdx,$sdy) = $path->n_to_dxdy($n-2);
      my ($tdx,$tdy) = $path->n_to_dxdy($n+1);
      my $sx = $x . chamfer(-$sdx,$MyOverlapGap);
      my $sy = $y . chamfer(-$sdy,$MyOverlapGap);
      my $tx = $x . chamfer($tdx,$MyOverlapGap);
      my $ty = $y . chamfer($tdy,$MyOverlapGap);
      printwrap("--($sx,$sy)");
      printwrap("--($tx,$ty)");
      push @overlap_circles,"$x,$y";

    } elsif (is_overlap($n,$n_hi)) {
      # start of an overlap
      if (is_overlap($n+1,$n_hi)) {
        $x = $x . chamfer(-$prev_dx,$MyOverlapGap);
        $y = $y . chamfer(-$prev_dy,$MyOverlapGap);
      }
      printwrap("--($x,$y)");

    } elsif (is_overlap($n-1,$n_hi)) {
      # end of an overlap
      my ($dx,$dy) = $path->n_to_dxdy($n);
      $x = $x . chamfer($dx,$MyOverlapGap);
      $y = $y . chamfer($dy,$MyOverlapGap);
      printwrap("--($x,$y)");

    } elsif ($dx == $prev_dx && $dy == $prev_dy) {
      # straight line, no new point

    } else {
      printwrap("--($x,$y)");
    }
  }
  printwrap("--(end-$level);\n");

  if (@overlap_circles) {
    $Printwrap = 4;
    printwrap("\\fill foreach \\p in {");
    foreach my $xy (@overlap_circles) {
      printwrap("($xy),");
    }
    printwrap("}\n");
    print "      { \\p circle[radius=$MyOverlapGap] };\n";
  }

  $Printwrap = 2;
  printwrap("\\end{scope}\n");
}

print "  % end generated\n";
exit 0;
