#!/usr/bin/perl -w

# Copyright 2015, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;

BEGIN { unshift @INC, $FindBin::Bin }
use MyPrintwrap ':all';
use MyCCurve 'chamfer';


print "  % generated by tools/$FindBin::Script\n";

my $s = 2;

# dots
$Printwrap = 2;
foreach my $y (0 .. $s) {
  foreach my $x (0 .. $s) {
    printwrap("\\node at ($x,$y) [my dot];");
  }
  printwrap_newline();
}

# horizontals
foreach my $y (0 .. $s) {
  {
    my $x = 0;
    printwrap(" \\draw      ($x-\\MyShorten,$y+\\MyGap) -- ($x-\\MyEnd, $y+\\MyGap);");
    printwrap(" \\draw [->] ($x-\\MyShorten,$y-\\MyGap) -- ($x-\\MyEnd, $y-\\MyGap);");
  }
  foreach my $x (0 .. $s-1) {
    my $x2 = $x + 1;
    printwrap(" \\draw [<-] ($x+\\MyShorten,$y+\\MyGap) -- ($x2-\\MyShorten,$y+\\MyGap);");
    printwrap(" \\draw [->] ($x+\\MyShorten,$y-\\MyGap) -- ($x2-\\MyShorten,$y-\\MyGap);");
  }
  {
    my $x = $s;
    printwrap(" \\draw [<-] ($x+\\MyShorten,$y+\\MyGap) -- ($x+\\MyEnd, $y+\\MyGap);");
    printwrap(" \\draw      ($x+\\MyShorten,$y-\\MyGap) -- ($x+\\MyEnd, $y-\\MyGap);");
  }
}

# verticals
foreach my $x (0 .. $s) {
  {
    # bottom end
    my $y = 0;
    printwrap(" \\draw      ($x+\\MyGap,$y-\\MyShorten) -- ($x+\\MyGap,$y-\\MyEnd);");
    printwrap(" \\draw [<-] ($x-\\MyGap,$y-\\MyShorten) -- ($x-\\MyGap,$y-\\MyEnd);");
  }
  foreach my $y (0 .. $s-1) {
    my $y2 = $y + 1;
    printwrap(" \\draw [->] ($x+\\MyGap,$y+\\MyShorten) -- ($x+\\MyGap,$y2-\\MyShorten);");
    printwrap(" \\draw [<-] ($x-\\MyGap,$y+\\MyShorten) -- ($x-\\MyGap,$y2-\\MyShorten);");
  }
  {
    # upper end
    my $y = $s;
    printwrap(" \\draw [->] ($x+\\MyGap,$y+\\MyEnd) -- ($x+\\MyGap,$y+\\MyShorten);");
    printwrap(" \\draw      ($x-\\MyGap,$y+\\MyEnd) -- ($x-\\MyGap,$y+\\MyShorten);");
  }
}

printwrap_newline();
$Printwrap = 2;
printwrap("\\begin{scope}[shift=(right-pos)]\n");

# dots
$Printwrap = 4;
foreach my $y (0 .. $s) {
  foreach my $x (0 .. $s) {
    printwrap(" \\node (p$x-$y) at ($x,$y) [my dot];");
  }
  printwrap_newline();
}

# horizontals
foreach my $y (0 .. $s) {
  {
    # left end
    my $x = -1;
    my $x2 = $x+1;
    {
      my $m1 = "$x+.5-\\MyChamfer,$y+.5-\\MyGapTwo-\\MyChamfer";
      my $m2 = "$x+.5+\\MyChamfer,$y+.5-\\MyGapTwo-\\MyChamfer";
      printwrap(" \\draw ($m1) -- ($m2);");
      printwrap(" \\draw [<-] ($m2) -- ($x2-\\MyGapTwo-\\MyShortenTwo,$y+\\MyShortenTwo);");
      # printwrap(" \\draw [<-] ($x-\\MyEndTwo,$y+\\MyEndTwo-\\MyGapTwo) -- ($x-\\MyShortenTwo-\\MyGapTwo,$y+\\MyShortenTwo);");
    }
    {
      my $m1 = "$x+.5-\\MyChamfer,$y-.5+\\MyGapTwo+\\MyChamfer";
      my $m2 = "$x+.5+\\MyChamfer,$y-.5+\\MyGapTwo+\\MyChamfer";
      printwrap(" \\draw ($m1) -- ($m2);");
      printwrap(" \\draw [->] ($m2) -- ($x2-\\MyGapTwo-\\MyShortenTwo,$y-\\MyShortenTwo);");
      # printwrap(" \\draw [->] ($x-\\MyEndTwo,$y-\\MyEndTwo+\\MyGapTwo) -- ($x-\\MyShortenTwo-\\MyGapTwo,$y-\\MyShortenTwo);");
    }
  }
  foreach my $x (0 .. $s-1) {
    my $x2 = $x+1;
    {
      # above
      my $m1 = "$x+.5-\\MyChamfer,$y+.5-\\MyGapTwo-\\MyChamfer";
      my $m2 = "$x+.5+\\MyChamfer,$y+.5-\\MyGapTwo-\\MyChamfer";
      printwrap(" \\draw [<-] ($x+\\MyGapTwo+\\MyShortenTwo,$y+\\MyShortenTwo) -- ($m1);");
      printwrap(" \\draw ($m1) -- ($m2);");
      printwrap(" \\draw [<-] ($m2) -- ($x2-\\MyGapTwo-\\MyShortenTwo,$y+\\MyShortenTwo);");
    }
    {
      # below
      my $m1 = "$x+.5-\\MyChamfer,$y-.5+\\MyGapTwo+\\MyChamfer";
      my $m2 = "$x+.5+\\MyChamfer,$y-.5+\\MyGapTwo+\\MyChamfer";
      printwrap(" \\draw [->] ($x+\\MyGapTwo+\\MyShortenTwo,$y-\\MyShortenTwo) -- ($m1);");
      printwrap(" \\draw ($m1) -- ($m2);");
      printwrap(" \\draw [->] ($m2) -- ($x2-\\MyGapTwo-\\MyShortenTwo,$y-\\MyShortenTwo);");
    }
  }
  {
    # right end
    my $x = $s;
    {
      # above
      my $m1 = "$x+.5-\\MyChamfer,$y+.5-\\MyGapTwo-\\MyChamfer";
      my $m2 = "$x+.5+\\MyChamfer,$y+.5-\\MyGapTwo-\\MyChamfer";
      printwrap(" \\draw [<-] ($x+\\MyGapTwo+\\MyShortenTwo,$y+\\MyShortenTwo) -- ($m1);");
      printwrap(" \\draw ($m1) -- ($m2);");
    }
    {
      # below
      my $m1 = "$x+.5-\\MyChamfer,$y-.5+\\MyGapTwo+\\MyChamfer";
      my $m2 = "$x+.5+\\MyChamfer,$y-.5+\\MyGapTwo+\\MyChamfer";
      printwrap(" \\draw [->] ($x+\\MyGapTwo+\\MyShortenTwo,$y-\\MyShortenTwo) -- ($m1);");
      printwrap(" \\draw ($m1) -- ($m2);");
    }
    # printwrap(" \\draw [<-] ($x+\\MyShortenTwo+\\MyGapTwo,$y+\\MyShortenTwo) -- ($x+\\MyEndTwo,$y+\\MyEndTwo-\\MyGapTwo);");
    # printwrap(" \\draw [->] ($x+\\MyShortenTwo+\\MyGapTwo,$y-\\MyShortenTwo) -- ($x+\\MyEndTwo,$y-\\MyEndTwo+\\MyGapTwo);");
  }
  printwrap_newline();
}

# verticals
foreach my $x (0 .. $s) {
  {
    my $y = -1;
    my $y2 = $y+1;
    {
      # right
      my $m1 = "$x+.5-\\MyGapTwo-\\MyChamfer,$y+.5-\\MyChamfer";
      my $m2 = "$x+.5-\\MyGapTwo-\\MyChamfer,$y+.5+\\MyChamfer";
      printwrap(" \\draw ($m1) -- ($m2);");
      printwrap(" \\draw [->] ($m2) -- ($x+\\MyShortenTwo,$y2-\\MyGapTwo-\\MyShortenTwo);");
    }
    {
      # left
      my $m1 = "$x-.5+\\MyGapTwo+\\MyChamfer,$y+.5-\\MyChamfer";
      my $m2 = "$x-.5+\\MyGapTwo+\\MyChamfer,$y+.5+\\MyChamfer";
      printwrap(" \\draw ($m1) -- ($m2);");
      printwrap(" \\draw [<-] ($m2) -- ($x-\\MyShortenTwo,$y2-\\MyGapTwo-\\MyShortenTwo);");
    }
  }
  foreach my $y (0 .. $s-1) {
    my $y2 = $y+1;
    {
      # right
      my $m1 = "$x+.5-\\MyGapTwo-\\MyChamfer,$y+.5-\\MyChamfer";
      my $m2 = "$x+.5-\\MyGapTwo-\\MyChamfer,$y+.5+\\MyChamfer";
      printwrap(" \\draw [->] ($x+\\MyShortenTwo,$y+\\MyGapTwo+\\MyShortenTwo) -- ($m1);");
      printwrap(" \\draw ($m1) -- ($m2);");
      printwrap(" \\draw [->] ($m2) -- ($x+\\MyShortenTwo,$y2-\\MyGapTwo-\\MyShortenTwo);");
    }
    {
      # left
      my $m1 = "$x-.5+\\MyGapTwo+\\MyChamfer,$y+.5-\\MyChamfer";
      my $m2 = "$x-.5+\\MyGapTwo+\\MyChamfer,$y+.5+\\MyChamfer";
      printwrap(" \\draw [<-] ($x-\\MyShortenTwo,$y+\\MyGapTwo+\\MyShortenTwo) -- ($m1);");
      printwrap(" \\draw ($m1) -- ($m2);");
      printwrap(" \\draw [<-] ($m2) -- ($x-\\MyShortenTwo,$y2-\\MyGapTwo-\\MyShortenTwo);");
    }
  }
  {
    my $y = $s;
    my $y2 = $y+1;
    {
      # right
      my $m1 = "$x+.5-\\MyGapTwo-\\MyChamfer,$y+.5-\\MyChamfer";
      my $m2 = "$x+.5-\\MyGapTwo-\\MyChamfer,$y+.5+\\MyChamfer";
      printwrap(" \\draw [->] ($x+\\MyShortenTwo,$y+\\MyGapTwo+\\MyShortenTwo) -- ($m1);");
      printwrap(" \\draw ($m1) -- ($m2);");
    }
    {
      # left
      my $m1 = "$x-.5+\\MyGapTwo+\\MyChamfer,$y+.5-\\MyChamfer";
      my $m2 = "$x-.5+\\MyGapTwo+\\MyChamfer,$y+.5+\\MyChamfer";
      printwrap(" \\draw [<-] ($x-\\MyShortenTwo,$y+\\MyGapTwo+\\MyShortenTwo) -- ($m1);");
      printwrap(" \\draw ($m1) -- ($m2);");
    }
  }
  printwrap_newline();
}

print "  \\end{scope}\n";
print "  % end generated\n";
exit 0;
